from django.conf import settings
from .excs import *
from django.utils.importlib import import_module
from django.core.exceptions import ImproperlyConfigured
import copy


class KwargsMixin(object):
	'''
	every keyword argument would be attached to the class instance itself with an 
	attribute whose name is the keyword argument
	'''
	def __init__(self, **kwargs):
		for key in kwargs.keys():
			setattr(self, key, kwargs.get(key, None))


# length of a bucket dict
def len_bdict(a):
	c = 0
	for key, list_ in a.items():
		if len(list_) > 0: c += 1
	return c

# caches
c_s = {} # count of fields to a form
ft_cache = {} # field types cache 

def get_ft(s):
	a = "%s_ft" % s
	if ft_cache.get(a, None):
		return ft_cache[a]

	_default_fts = ["formt.fts", ]
	b = list(getattr(settings, 'FORMS_FT', list())) + _default_fts

	for _c in b:
		try:
			c = import_module(_c)
		except ImportError as e:
			raise ImproperlyConfigured('field type location "%s" not importable' % _c)

		if not hasattr(c, a):
			continue

		ft_cache[a] = getattr(c, a)

		break

	if not a in ft_cache:
		raise ImproperlyConfigured('no ft: "%s"' % a)
	return ft_cache[a]

ft = get_ft

def _add_pynames(f):
	for a, b in f.items():
		b.sname = a


def former(fields, meta, vs=[], deffunc=None):
	fields = copy.deepcopy(fields)
	
	if not "n" in meta:
		raise BadForm('form must have a name')
	if deffunc != None and not callable(deffunc):
		raise BadForm('deffunc must be a callable')

	class form(KwargsMixin):
		def __str__(self):
			return "form: " + meta['n']

	c_s[ meta['n'] ] = 0

	_add_pynames(fields)

	return form(fields=fields, meta=meta, vs=vs, deffunc=deffunc)


def form_iter(f):
	return iter(f.fields.values())


# addition of fields to a form after the form has been built
def add_field(a, b):
	c = len(a.fields)
	d = copy.deepcopy(b)
	_add_pynames(d)
	a.fields.update(d)


def field(label="", n="", ft=None):
	if not ft:
		ft = get_ft("string")

	class _field(KwargsMixin):
		def __repr__(self):
			cls = self.__class__.__module__ + '.' + self.__class__.__name__
			return '<' + cls + ": label=%s" % self.label + '>'

	# in cases where the type haven't been initialized (if no arguments is passed in during form definition)
	if isinstance(ft, type):
		ft = ft()

	if c_s.get(n, None) in (0, None):
		c_s[n] = 1	# 1-index for sanity
	ret = _field(label=label, n=n, c=c_s[n], ft=ft)
	c_s[n] += 1
	return ret


# must be run after validation cause some of the methods it calls "f.ft.get_post_validation_data" can throw
def get_form_data(form, data):
	d = {}
	for f in form_iter(form):
		e = None
		if getattr(form, 'deffunc', None) != None:
			e = form.deffunc(form, f.sname)
		g = f.ft.get_post_validation_data(f.sname, data, default=e)
		d[f.sname] = g
	return d


def validate_form(form, data):
	field_err_msgs, form_err_msgs = {}, []

	# field validators
	for a in form_iter(form):
		b = None
		if getattr(form, 'deffunc', None) != None:
			b = form.deffunc(form, a.sname)
		c = a.ft.get_pre_validation_data(a.sname, data, default=b)

		for e in a.ft.validators():
			f = e.validate(c)

			if f is None:
				continue
			else:
				field_err_msgs[a.sname] = [ e.err_msg(c) ]
				break # makes sure only one validator is ran, some others can throw error if data is not well prepped
	
	# form validators
	for h in form.vs:
		i = get_form_data(form, data)
		h.form_data = i
		j = h.validate()
		if j is None:
			continue
		else:
			form_err_msgs.append( h.err_msg() )
			break # makes sure only one validator is ran, some others can throw error if data is not well prepped

	# return validation results
	if len(form_err_msgs) > 0 or len_bdict(field_err_msgs) > 0:
		return False, field_err_msgs, form_err_msgs
	return True, field_err_msgs, form_err_msgs

