from . import validators as v
from .fts import _base_ft
from .excs import *


class range_v(v._base_v):
	errcode = 103
	msg = u'value of subfield {subfield} not in range [{min}, {max}]'

	def validate(self, value):
		if value[str(self.subfield)] > self.max or value[str(self.subfield)] < self.min:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(subfield = self.subfield, max=self.max, min=self.min, value=value, errcode=self.errcode)

class range_max_v(v._base_v):
	errcode = 104
	msg = u'value of subfield {subfield} is greater than the maximum value ({max})'

	def validate(self, value):
		if value[str(self.subfield)] > self.max:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(subfield=self.subfield, max=self.max, errcode=self.errcode)


class range_min_v(v._base_v):
	errcode = 105
	msg = u'value of subfield {subfield} is less than the minimum value ({min})'

	def validate(self, value):
		if value[ str(self.subfield) ] < self.min:
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(subfield=self.subfield, min=self.min, errcode=self.errcode)


class range_int_v(v._base_v):
	errcode = 105
	msg = u'some value(s) not convertible to int'

	def validate(self, value):
		try:
			a = int(str(value['1']))
			b = int(str(value['2']))
		except (ValueError, TypeError):
			return self.errcode

	def err_msg(self, value):
		return self.msg.format(errcode=self.errcode)


class range_ft(_base_ft):
	max_1 = None
	max_2 = None
	min_1 = None
	min_2 = None
	required = False

	def get_pre_validation_data(self, n, data, default=None):
		d = {'1': data.get(n + '-1', None),  # 1 and 2 is used to avoid attaching meaning to subfields
			'2': data.get(n + '-2', None)}
		for k, v in d.items():
			if len(d[k]) == 0:
				if default != None and k == '1':
					d[k] = default['1']
				elif default != None and k == '2':
					d[k] = default['2']
		return d

	def get_post_validation_data(self, n, data, default=None):
		d = self.get_pre_validation_data(n, data, default=default)
		for k, v in d.items():
			d[k] = int( str(v) )
		return d

	def validators(self):
		vs = [ range_int_v(), ]
		if self.required:
			vs.append(range_required_v(required=self.required))


		if self.max_1 != None and self.min_1 != None:
			vs.append(range_v(max=self.max_1, min = self.min_1, subfield=1))
		else:
			if self.min_1 != None:
				vs.append(range_min_v(min=self.min_1, subfield=1))
			if self.max_1 != None:
				vs.append(range_max_v(max=self.max_1, subfield=1))

		if self.max_2 != None and self.min_2 != None:
			vs.append(range_v(max=self.max_2, min=self.min_2, subfield=2))
		else:
			if self.min_2 != None:
				vs.append(range_min_v(min=self.min_2, subfield=2))
			if self.max_2 != None:
				vs.append(range_max_v(max=self.max_2, subfield=2))
		
		return vs




					
