from . import *
from django.utils.safestring import mark_safe

_ft_widgets = {}

def register_ftwidget(ft, widget_func):
	_ft_widgets[ft.__name__] = widget_func


def get_ftwidget(ft):
	if isinstance(ft, type):
		return _ft_widgets[ft.__name__]
	else:
		return _ft_widgets[ft.__class__.__name__]


def string_widget(form, a, d=(True, dict(), list())):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""
	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += '<span>%s</span><br />' % s
	input_ += '<input type="text" id="id_{0}" name="{0}" value="" />'.format(a.sname)
	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)


def int_widget(form, a, d=(True, dict(), list())):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""
	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += '<span>%s</span><br />' % s
	input_ += '<input type="number" id="id_{0}" name="{0}" value="" />'.format(a.sname)
	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)


register_ftwidget(ft("string"), string_widget)
register_ftwidget(ft("int"), int_widget)

def range_widget(form, a, d=(True, dict(), list())):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""
	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += u'<span>%s</span><br />' % s

	input_ += '<input type="text" name="{name}-1" value="" />'
	input_ += '<input type="text" name="{name}-2" value="" />'
	input_ = input_.format(name=a.sname)

	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)


# without the try block spits an error in projects that don't have range_ft configured
try:
	register_ftwidget(ft('range'), range_widget)
except Exception:
	pass


def bool_widget(form, a, d=(True, {}, []) ):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""
	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += u'<span>%s</span><br />' % s

	if form.deffunc(form, a.sname) == 'on':
		input_ += '<input type="checkbox" checked="checked" name="{name}" />{question}'    
	else:
		input_ += '<input type="checkbox" name="{name}" />{question}'
	input_ = input_.format(name=a.sname, question=a.ft.question)

	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)

register_ftwidget(ft('bool'), bool_widget)


def choice_widget(form, a, d=(True, {}, []) ):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""

	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += '<span>%s</span><br />' % s

	for e, f in a.ft.choices:
		if e == form.deffunc(form, a.sname):
			input_ += '<input type="radio" name="{name}" checked="checked" value="{value}" />{label}<br />'.format(name=a.sname, value=e, label=f)
		else:
			input_ += '<input type="radio" name="{name}" value="{value}" />{label}<br />'.format(name=a.sname, value=e, label=f)

	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)

register_ftwidget(ft('choice'), choice_widget)


def multi_choice_widget(form, a, d=(True, {}, []) ):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""

	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += '<span>%s</span><br />' % s

	for e, f in a.ft.choices:
		g = form.deffunc(form, a.sname)
		if g != None and e in g:
			input_ += '<input type="checkbox" name="{name}" checked="checked" value="{value}" />{label}<br />'.format(name=a.sname, value=e, label=f)
		else:
			input_ += '<input type="checkbox" name="{name}" value="{value}" />{label}<br />'.format(name=a.sname, value=e, label=f)

	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)

register_ftwidget(ft('multi_choice'), multi_choice_widget)


def date_widget(form, a, d=(True, dict(), list())):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""
	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += '<span>%s</span><br />' % s
	input_ += '<input type="date" id="id_{0}" name="{0}" value="" />'.format(a.sname)
	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)

register_ftwidget(ft('date'), date_widget)


def time_widget(form, a, d=(True, dict(), list())):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""
	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += '<span>%s</span><br />' % s
	input_ += '<input type="time" id="id_{0}" name="{0}" value="" />'.format(a.sname)
	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)

register_ftwidget(ft('time'), time_widget)


def datetime_widget(form, a, d=(True, dict(), list())):
	validation_result, field_errs, form_errs = d[0], d[1], d[2]

	label = '<label>%s:</label>' % a.label
	input_ = ""
	if not validation_result and a.sname in field_errs and len(field_errs[a.sname]) > 0:
		for i, s in enumerate(field_errs[a.sname]):
			input_ += '<span>%s</span><br />' % s
	input_ += '<input type="datetime" id="id_{0}" name="{0}" value="" />'.format(a.sname)
	t = '<div>%s%s</div>' % (label, input_)
	return mark_safe(t)

register_ftwidget(ft('datetime'), datetime_widget)


def form_string(f, d=(True, dict(), list())):
    
	b = u''
	for a in form_iter(f):
		c = getattr(a, "ft", ft("string")) # get field type
		widget_func = get_ftwidget(c)
		b += widget_func(f, a, d)
	return b
